#!/usr/bin/python3

import re
import json
import sys
from os.path import exists


########### read adventure file name

if ( len(sys.argv) != 2 ):
    print('Fehler: bitte mit Filename (ohne .TXT) aufrufen')
    exit()

fn = sys.argv[1]
print('Filename: %s' % fn)

if ( not exists('%s.txt' % fn) ):
    print('Datei %s.txt existiert nicht' % fn)
    exit()


########### read adventure file
with open('%s.txt' % fn , 'r') as file:
    Lines = file.readlines()

with open('instode_vorlage.html', 'r') as file:
    vorlage = file.read()


# DSL match strings

#rx_header = re.compile('==+ ?([äÄüÜöÖßA-Za-z0-9._-]+) ==+')
rx_header = re.compile('==+ ?([ äÄüÜöÖßA-Za-z0-9._-]+) ==+')
rx_comment = re.compile('~~')
rx_return = re.compile('<< ?RETURN ?(.*)')
rx_click = re.compile('<> ?([äÄüÜöÖßA-Za-z0-9_-]+) (.*)')
rx_jump = re.compile('>> ?([äÄüÜöÖßA-Za-z0-9_-]+) (.*)')
rx_condjump = re.compile('\? ?([äÄüÜöÖßA-Za-z0-9._-]+) ?>> ?([äÄüÜöÖßA-Za-z0-9._-]+) (.*)')
rx_notjump = re.compile('! ?([äÄüÜöÖßA-Za-z0-9._-]+) ?>> ?([äÄüÜöÖßA-Za-z0-9._-]+) (.*)')
rx_FIN = re.compile('__FIN__')


########### 1st parse run: read nodes

# initial
linecount = 0
nodecount = 0
nodes = ['startmuell_please_ignore']
nodenr = {'startmuell_please_ignore':0}
nodeused = {0:True, 1:True}

# nodes[NR] => Node-Titel
# nodenr["titel"] => Node-Nr
# nodeused[NR] => true/false
for line in Lines:
    linecount += 1
    justtext = True
    match = rx_header.match(line)
    if match:
        nodecount += 1
        hdr = match.group(1)
        if hdr in nodes:
            print( "FEHLER: Header doppelt verwendet Zeile %d - Header %s" % (linecount, hdr ) )
#        else:
#            print( "Zeile %d - Header %d: %s" % (linecount, nodecount, hdr ) )
        nodes.append(hdr)
        nodenr[hdr] = nodecount
        nodeused[nodecount] = False

print("%d Nodes\n" % nodecount )
#print("-------------------------------------\n")


########### 2nd parse run: parse file into (json) structure

# initial
jsstruct = {}
linecount = 0
nodeused[2] = True
txt = ''
hdr = ''
hdrnr = 0
graphLog = "digraph adv {\n"
graphLogAnon = "digraph adv {\n"
jumps = []
final = False
hdr = 'startmuell_please_ignore'
rx_link = re.compile('(https?://[a-zA-Z0-9:$%&?=.+_/-]*)')
rx_email = re.compile('([a-zA-Z0-9._-]+\@[a-zA-Z0-9._-]+)')

# Reset
nodecount = 0
linecount = 0

for rawline in Lines:
    linecount += 1
    line = rawline.replace("'", '&quot;')
    justtext = True

    match = rx_comment.match(line)
    if match:
        justtext = False

    match = rx_FIN.match(line)
    if match:
        final = True
        justtext = False

    match = rx_return.match(line)
    if match:
        justtext = False
        substruct = {}
        substruct['if'] = []
        substruct['notif'] = []
        substruct['target'] = 0
        substruct['text'] = match.group(1).strip()
        jumps.append(substruct)

    match = rx_jump.match(line)
    if match:
        justtext = False
        substruct = {}
        substruct['if'] = []
        substruct['notif'] = []
        target = match.group(1)
        targetnr = nodenr.get( target )
        if targetnr:
            nodeused[targetnr] = True
            substruct['target'] = targetnr
            substruct['text'] = match.group(2).strip()
            jumps.append(substruct)
            graphLogAnon += "    %s -> %s;\n" % ( hdrnr, targetnr )
            graphLog += "    %s -> %s;\n" % ( hdr, target )
        else:
            print( "Sprungziel unbekannt: Zeile %d (%s) - %s" % (linecount, hdr, target) )

    match = rx_condjump.match(line)
    if match:
        justtext = False
        substruct = {}
        condnode = match.group(1)
        condnr = nodenr.get( condnode )
        if condnr:
            substruct['if'] = [ condnr ]
            substruct['notif'] = []
            target = match.group(2)
            targetnr = nodenr.get( target )
            if targetnr:
                nodeused[targetnr] = True
                substruct['target'] = targetnr
                substruct['text'] = match.group(3).strip()
                jumps.append(substruct)
                graphLog += "    %s -> %s [label=if];\n" % ( hdr, target )
                graphLogAnon += "    %s -> %s [label=if];\n" % ( hdrnr, targetnr )
            else:
                print( "Sprungziel unbekannt: Zeile %d (%s) - %s" % (linecount, hdr, target) )
        else:
            print( "Bedingung unbekannt: Zeile %d (%s) - %s" % (linecount, hdr, condnode) )

    match = rx_notjump.match(line)
    if match:
        justtext = False
        substruct = {}
        condnode = match.group(1)
        condnr = nodenr.get( condnode )
        if condnr:
            substruct['if'] = []
            substruct['notif'] = [ condnr ]
            target = match.group(2)
            targetnr = nodenr.get( target )
            if targetnr:
                nodeused[targetnr] = True
                substruct['target'] = targetnr
                substruct['text'] = match.group(3).strip()
                jumps.append(substruct)
                graphLog += "    %s -> %s [label=not];\n" % ( hdr, target )
                graphLogAnon += "    %s -> %s [label=not];\n" % ( hdrnr, targetnr )
            else:
                print( "Sprungziel unbekannt: Zeile %d (%s) - %s" % (linecount, hdr, target) )
        else:
            print( "Bedingung unbekannt: Zeile %d (%s) - %s" % (linecount, hdr, condnode) )

    match = rx_header.match(line)
    if match:
        justtext = False
        substruct = {}

        txtlen = len(txt)

        # Überschriften ignorieren wir
        if ( (jumps == [] ) and (txtlen < 4) and not final ):
                print( "Nur Überschrift?  Zeile %d: %s " % (linecount, hdr) )
        else:
            if txtlen < 40:
                print( "Hinweis: der Text des Nodes ist mit nur %d Zeichen sehr kurz, Zeile %d: %s " % (txtlen, linecount, hdr) )
            substruct['text'] = txt

            substruct['nodenr'] = nodenr[hdr]
            substruct['titel'] = hdr
            substruct['branches'] = jumps
            substruct['final'] = final
            jsstruct[nodenr[hdr]] = substruct
            if ( (jumps == [] ) and (nodenr[hdr] > 1) and not final ):
                print( "Fehler: Node hat weder Verzweigungen noch ist final, Zeile %d: %s " % (linecount, hdr) )

            if ( (hdr == '0_START') and (nodenr[hdr] != 2)):
                print( "Fehler: 0_START ist nicht der zweite Node (nach der Ueberschrift), sondern der %d." % nodenr[hdr] )
            else:
                nodeused[1] = True

        txt = ''
        jumps = []
        final = False
        hdr = match.group(1)
        hdrnr = nodenr[hdr]

    if justtext:
        line = rx_link.sub(r'<a href="\1" target="_blank">\1</a>', line )
        line = rx_email.sub(r' <a href="mailto:\1" target="_blank">\1</a>', line )
        txt += line.replace("\n",' <br/>')


########### check for unused nodes

for hdr in nodes:
    nr = nodenr[hdr]
    if (not nodeused[nr]) and (nr < nodecount):
        print("Node nicht angesprungen: %d = %s" % (nr,hdr) )

if ( hdr != 'FIN' ):
    print( "Fehler: Dokument endet nicht mit FIN als letzem (leeren) Node, sondern mit %s." % hdr )



########### output

### JSON
#
with open('%s.json' % fn, 'w') as outfile:
    json.dump(jsstruct, outfile)


### HTML

jsstring = json.dumps(jsstruct)
vorlage1 = vorlage.replace('<<<TITEL>>>', nodes[1]) 
with open('%s.html' % fn, 'w') as outfile:
    outfile.write( vorlage1.replace('§§§JSON§§§', jsstring) )

### DOT

graphLog += '}'
with open('%s.dot' % fn, 'w') as outfile:
    outfile.write( graphLog )

graphLogAnon += '}'
with open('%s_anon.dot' % fn, 'w') as outfile:
    outfile.write( graphLogAnon )
